'From Squeak3.2gamma of 15 January 2002 [latest update: #4843] on 6 May 2002 at 2:12:32 pm'!

!Preferences commentStamp: '<historical>' prior: 0!
A general mechanism to store preference choices.  The default setup treats any symbol as a potential boolean flag; flags unknown to the preference dictionary are always returned as false.  

	To open the control panel:		Preferences openFactoredPanel
	To read how to use the panel (and how to make a preference be per-project):
		 Preferences giveHelpWithPreferences

All messages are on the class side.

To query a a preference:
	Preferences logDebuggerStackToFile
or some people prefer the more verbose
	Preferences valueOfFlag: #logDebuggerStackToFile

You can make up a new preference any time.  Do not define a new message in Preferences class. Accessor methods are compiled automatically when you add a preference as illustrated below:

To add a preference (e.g. in the Postscript of a fileout):
	Preferences addPreference: #samplePreference categories: #(general browsing)
		default: true balloonHelp: 'This is an example of a preference added by a do-it'
		projectLocal: false changeInformee: nil changeSelector: nil.

To remove a preference:
	Preferences expungePreferenceNamed: #samplePreference

To change a preference programatically:
	Preferences disable: #logDebuggerStackToFile.
Or to turn it on,
	Preferences enable: #logDebuggerStackToFile.
!
]style[(220 29 81 35 893)f1,f1dPreferences openFactoredPanel;;,f1,f1dPreferences giveHelpWithPreferences;;,f1!

Object subclass: #SocketRelay
	instanceVariableNames: 'client server inPort outPort delegate log process '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Network-Kernel'!

!SocketRelay commentStamp: '<historical>' prior: 0!
A SocketRelay is an object that listens on one port using a Socket and forwards all incoming traffic to another machine and port. It "sits" inbetween and is useful for examining and debugging Socket protocols.

You can set one up for examining CVS (normally uses port 2401) by calling:

Smalltalk at: #MyRelay put: (SocketRelay forwardPort: 2041 toPort: 2041 at: 'www.bluefish.se' loggingTo: Transcript).

Right now you have to stop/start it between every session:

MyRelay start
MyRelay stop!

SocketRelay class
	instanceVariableNames: ''!

!ConfigFile methodsFor: 'private' stamp: 'JO 3/4/2002 14:58'!
addKey: aKeyString value: aValueString onStream: aFileStream
"Add a key to the topic -- assumes aFileStream has already been positioned after the topic string"

	|   tempStream  |

	"Create the output string"
	(tempStream _ WriteStream on: String new)
"		crlf;"
		nextPutAll: aKeyString; 
		nextPut: $=;
		nextPutAll: (self configFileForm: aValueString); 
		crlf.
	tempStream close.

	aFileStream
		spliceString: tempStream contents
		from: aFileStream position
		to: aFileStream position.! !

!ConfigFile methodsFor: 'private' stamp: 'JO 3/4/2002 14:58'!
addTopic: aTopicString onStream: aFileStream
		"Add the new topic to the end of the file"

	aFileStream 
		position: aFileStream size;
		crlf;
		nextPut: $[;
		nextPutAll: aTopicString;
		nextPut: $];
		crlf.! !

!ConfigFile methodsFor: 'private' stamp: 'JO 3/4/2002 13:33'!
readFromFile
"Read in the config file and cache all the values in the topicDict"

	| fileStream topic curTopicDict keyVal |

	self topicDict: Dictionary new.
	fileStream _ self fileStream.
	[ 
		[ (topic _ self getNextTopic: fileStream) isNil ]  whileFalse:[

			self topicDict at: topic put: (curTopicDict _ Dictionary new).

			[ (keyVal _ self getNextKeyValue: fileStream) isNil ] whileFalse: [
				curTopicDict add: keyVal.	
			]
		]
"	] ensure: "
	] ifError:
		[ 
			fileStream close. 
			^false 
		].

	fileStream close.
	^self topicDict! !


!FileDirectory methodsFor: 'file operations' stamp: 'JO 4/16/2002 17:27'!
deleteDirectoryTree: localDirName
	"Delete the directory and its contents with the given name in this directory. Fail if the path is bad or if a directory by that name does not exist."

	| dir |
	dir _ self directoryNamed: localDirName.
	dir directoryNames do: [:dName |
		dir deleteDirectoryTree: dName].
	dir fileNames do: [:fName |
		dir deleteFileNamed: fName].
	self deleteDirectory: localDirName.
! !

!FileDirectory methodsFor: 'file name utilities' stamp: 'gh 1/22/2002 15:45'!
lastNameFor: baseFileName extension: extension
	"Assumes a file name includes a version number encoded as '.' followed by digits 
	preceding the file extension.  Increment the version number and answer the new file name.
	If a version number is not found, set the version to 1 and answer a new file name"

	| files splits |

	files _ self fileNamesMatching: (baseFileName,'*', self class dot, extension).
	splits _ files 
			collect: [:file | self splitNameVersionExtensionFor: file]
			thenSelect: [:split | (split at: 1) = baseFileName].
	splits _ splits asSortedCollection: [:a :b | (a at: 2) < (b at: 2)].
	^splits isEmpty 
			ifTrue: [nil]
			ifFalse: [(baseFileName, '.', (splits last at: 2) asString, self class dot, extension) asFileName]! !


!FileDirectory class methodsFor: 'create/delete file' stamp: 'JO 4/16/2002 15:32'!
deleteTreeForDir: dir
	dir directoryNames do: [:dName |
		self deleteTreeForDir: (dir directoryNamed: dName).
		dir deleteDirectory: dName].
	dir fileNames do: [:fName |
		dir deleteFileNamed: fName]! !


!HttpRequest methodsFor: 'initialize-release' stamp: 'gh 2/25/2002 20:30'!
initStatusString: aString
	| statusLine |
	statusLine _ aString findTokens: ' '.
	(statusLine size = 3 or: [statusLine size = 2])
		ifTrue: [self method: statusLine first.
			rawUrl _ statusLine second.
			url _ rawUrl copyUpTo: $?.
			"self queryString: (rawUrl copyFrom: (url size + 2) to: rawUrl size)."
			"self protocol: statusLine third"]
		ifFalse: [^nil]! !


!HttpSession methodsFor: 'accessing' stamp: 'gh 1/11/2002 16:29'!
timestamp
	^timestamp! !


!HttpSessionModule methodsFor: 'accessing' stamp: 'gh 1/11/2002 16:15'!
module
	^module! !

!HttpSessionModule methodsFor: 'private' stamp: 'gh 1/11/2002 16:17'!
sessionsDo: aBlock
	"Iterate over sessions."

	^ sessions valuesDo: aBlock! !


!ImageSegment methodsFor: 'read/write segment' stamp: 'gh 5/3/2002 12:06'!
install
	"This operation retrieves the segment if necessary from file storage, installs it in memory, and replaces (using become:) all the root stubs with the reconstructed roots of the segment."

	| newRoots |
	state = #onFile ifTrue: [self readFromFile].
	state = #onFileWithSymbols ifTrue: [self readFromFileWithSymbols.
		endMarker _ segment nextObject. 	"for enumeration of objects"
		endMarker == 0 ifTrue: [endMarker _ 'End' clone]].
	(state = #active) | (state = #imported) ifFalse: [self errorWrongState].
	newRoots _ self loadSegmentFrom: segment outPointers: outPointers.
	state = #imported 
		ifTrue: ["just came in from exported file"
			arrayOfRoots _ newRoots]
		ifFalse: [
			arrayOfRoots elementsForwardIdentityTo: newRoots].
	state _ #inactive.
	"self beep."	"Don't use Squeak sound here."
! !

!ImageSegment methodsFor: 'read/write segment' stamp: 'gh 1/22/2002 15:19'!
writeForExportOn: fileStream
	"Write the segment on the disk with all info needed to reconstruct it in a new image.  For export.  Out pointers are encoded as normal objects on the disk."

	| temp |
	state = #activeCopy ifFalse: [self error: 'wrong state'].
	temp _ endMarker.
	endMarker _ nil.
	fileStream fileOutClass: nil andObject: self.
		"remember extra structures.  Note class names."
	endMarker _ temp.
! !


!Socket methodsFor: 'waiting' stamp: 'JMM 5/22/2000 22:04'!
waitForDataUntil: deadline
	"Wait up until the given deadline for data to arrive. Return true if data arrives by the deadline, false if not."

	| dataArrived |
	[self isConnected & 
	 (dataArrived _ self primSocketReceiveDataAvailable: socketHandle) not
			"Connection end and final data can happen fast, so test in this order"
		and: [Time millisecondClockValue < deadline]] whileTrue: [
			self readSemaphore waitTimeoutMSecs: (deadline - Time millisecondClockValue)].

	^ dataArrived
! !


!Socket class methodsFor: 'utilities' stamp: 'gh 4/26/2002 11:49'!
quickDeadline
	^ self deadlineSecs: 3600! !


!SocketRelay methodsFor: 'private' stamp: 'gh 2/20/2000 20:57'!
connect
	"Create and connect sockets."

	| addr |
	server _ SimpleClientSocket new.
	addr _ NetNameResolver addressForName: delegate.
	addr ifNil: [ self error: 'Could not find address for ', delegate ].
	server connectTo: addr port: outPort.

	client _ SimpleClientSocket new.
	client listenOn: 2401.

	(server waitForConnectionUntil: Socket standardDeadline)
			ifFalse: [ self error: 'Connection failed' ].! !

!SocketRelay methodsFor: 'private' stamp: 'gh 2/22/2000 22:15'!
show: aString prefix: prefix
	"Log the traffic. Line by line using linefeed as delimiter."

	| lines |
	lines _ aString findTokens: (Array with: (Character lf)).
	lines do: [:line | log show: prefix, line; cr ]! !

!SocketRelay methodsFor: 'control' stamp: 'gh 2/20/2000 21:00'!
start
	"Start the relay."

	| buf bytes |
	self connect.
	process _ [
		buf _ String new: 50000.
		[server isConnected] whileTrue: [
			"poll the client"
			client dataAvailable ifTrue: [
				bytes _ client receiveDataInto: buf.
				self show: (buf copyFrom: 1 to: bytes) prefix: 'C:'.
				"send it to server"
				server sendData: buf count: bytes].

			"poll the server"
			server dataAvailable ifTrue: [
				bytes _ server receiveDataInto: buf.
				self show: (buf copyFrom: 1 to: bytes) prefix: 'S:'.
				"send it to client"
				client sendData: buf count: bytes ].
			(Delay forMilliseconds: 200) wait.
		]
	] fork! !

!SocketRelay methodsFor: 'control' stamp: 'gh 2/20/2000 20:08'!
stop
	"Stops the relay."

	process ifNotNil: [ process terminate ].
	process _ nil.
	client ifNotNil: [ client closeAndDestroy ].
	server ifNotNil: [ server closeAndDestroy ]
	! !

!SocketRelay methodsFor: 'accessing' stamp: 'gh 2/22/2000 22:16'!
delegate: aMachine
	"Set the delegate, the machine that we act as proxy for."

	delegate _ aMachine! !

!SocketRelay methodsFor: 'accessing' stamp: 'gh 2/19/2000 18:41'!
inPort: aPortNo
	"Set the port number."

	inPort _ aPortNo! !

!SocketRelay methodsFor: 'accessing' stamp: 'gh 2/22/2000 23:29'!
log: transcriptOrSomething
	"Set where we log traffic. It must support #show: and #cr,
	as Transcript does."

	log _ transcriptOrSomething! !

!SocketRelay methodsFor: 'accessing' stamp: 'gh 2/19/2000 18:41'!
outPort: aPortNo
	"Set the port number."

	outPort _ aPortNo! !


!SocketRelay class methodsFor: 'instance creation' stamp: 'gh 2/22/2000 22:15'!
forwardPort: inPort toPort: outPort at: delegate loggingTo: transcriptOrSomething 
	"Creates a SocketRelay."

	^ self new inPort: inPort;
	 outPort: outPort;
	 delegate: delegate;
	 log: transcriptOrSomething;
	 yourself! !


!SocketStream methodsFor: 'stream in' stamp: 'gh 5/2/2002 12:20'!
upTo: delim 
	| string |
	(inStream includes: delim)
		ifTrue: ["Found the delimiter part way into buffer"
			^inStream upTo: delim].
	self atEnd
		ifTrue: ["Never found it, and hit end of file"
			^self error: 'Can''t find delimiter, is atEnd.'. 
			"string _ inStream contents.
			self resetInStream.
			^string"].
	"Never found it, but there's more..."
	self pvtGetData.
	^self upTo: delim! !


!String methodsFor: 'accessing' stamp: 'gh 3/13/2000 22:23'!
findDelimitersBackwards: delimiters startingAt: start 
	"Answer the index of the character within the receiver, starting at start, that matches one of the delimiters. If the receiver does not contain any of the delimiters, answer 0."

	start to: 1 by: -1 do: [:i |
		delimiters do: [:delim | delim = (self at: i) ifTrue: [^ i]]].
	^0! !

!String methodsFor: 'converting' stamp: 'gh 1/28/2002 16:39'!
asHtmlSource
	"Do the basic character conversion for HTML.  Leave all original return 
	and tabs in place, so can conver back by simply removing bracked 
	things. 4/4/96 tk"
	| temp |
	temp _ self copyReplaceAll: '&' with: '&amp;'.
	HtmlEntities keysAndValuesDo:
		[:entity :char |
		char = $& ifFalse:
			[temp _ temp copyReplaceAll: char asString with: '&' , entity , ';']].
	^'<pre>', temp, '</pre>'

"
	'A<&>B' asHtml
"! !

!String methodsFor: 'Celeste' stamp: 'gh 11/8/2000 20:50'!
withCRLFs
	"Return a copy of the receiver in which backslash (\) characters have been replaced with <CR><LF>. Yep, pretty ugly..."

	| stream crlf |
	crlf _ String with: Character cr with: Character lf.
	stream _ WriteStream on: String new.
	self do: [ :c | c = $\ ifTrue: [ stream nextPutAll: crlf ] ifFalse: [ stream nextPut: c ]].
	^stream contents! !

!String methodsFor: 'Celeste' stamp: 'gh 1/7/2002 09:39'!
withLFs
	"Return a copy of the receiver in which backslash (\) characters have been replaced with <LF>. Yep, pretty ugly..."

	^self replaceAll: $\ with: Character lf! !


!TimeStamp class methodsFor: 'instance creation' stamp: 'JO 3/13/2002 14:26'!
fromSecondsPosix: anInteger
	"Answer a new instance based on the given number of seconds since 1 jan 1970."

	^self fromSeconds: (anInteger + self posixOffset)! !

!TimeStamp class methodsFor: 'instance creation' stamp: 'JO 3/13/2002 14:26'!
posixOffset
	"Logic is lifted from Ian's Unix support code.
	Squeak epoc	h is Jan 1, 1901.  Unix epoch is Jan 1, 1970: 17 leap years
     and 52 non-leap years later than Squeak."

	"52 * 365 + (17 * 366) * 24 * 60 * 60"

	^ 2177452800! !


!WriteStream methodsFor: 'character writing' stamp: 'gh 1/3/2002 16:45'!
crlf
	self nextPut: Character cr; nextPut: Character lf! !

!WriteStream methodsFor: 'character writing' stamp: 'gh 1/4/2002 14:08'!
lf
	"Append a linefeed character to the receiver."

	self nextPut: Character lf! !


!StandardFileStream methodsFor: 'read, write, position' stamp: 'gh 3/20/2002 00:08'!
findStringFromEnd: string
	"Fast version to find a String in a file starting from the end.
	Returns the position and also sets the position there.
	If string is not found 0 is returned and position is unchanged."

	| pos buffer count oldPos |
	oldPos _ self position.
	self setToEnd.
	pos _ self position.
	[ pos _ ((pos - 2000) max: 0).
	self position: pos.
	buffer _ self next: 2000.
	(count _ buffer findString: string) > 0
		ifTrue: ["Found the delimiter part way into buffer"
			self position: pos + count.
			^pos + count].
	pos = 0] whileFalse.
	"Never found it, and hit beginning of file"
	self position: oldPos.
	^0! !


!TranscriptStream methodsFor: 'access' stamp: 'gh 2/24/2000 20:14'!
characterLimit
	"Tell the views how much to retain on screen"
	^ 2000000! !

