'From Squeak3.2gamma of 15 January 2002 [latest update: #4843] on 13 May 2002 at 12:23:11 pm'!
HVHttpView subclass: #SMCardView
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SM'!
HVModule subclass: #SMModule
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SM'!

!SMModule commentStamp: 'gh 5/13/2002 12:22' prior: 0!
This is the Comanche module for the HttpView framework.
It holds onto the top domain object for the webapplication in the instvar 'model'.

Use these doits to start and stop SqueakMap.

"Create the top domain object once and for all"
Smalltalk at: #SqueakMap put: SMSqueakMap new

"Start up the service"
| service |
service := ComancheNetService named: 'SqueakMap' onPort: 8080.
service module: (SMModule on: SqueakMap prefix: '/sm').
service start

"And to stop it"
(ComancheNetService serviceNamed: 'SqueakMap') unregister

"Check which services are alive"
ComancheNetService services!

Object subclass: #SMObject
	instanceVariableNames: 'id created updated '
	classVariableNames: 'Ids '
	poolDictionaries: ''
	category: 'SM'!
SMObject subclass: #SMCard
	instanceVariableNames: 'name currentVersion summary description homePageUrl downloadUrl author maintainer registrator password modulePath categories '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SM'!
SMObject subclass: #SMCategory
	instanceVariableNames: 'name summary url mandatory '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SM'!
Object subclass: #SMSqueakMap
	instanceVariableNames: 'cards categories dir transactionCounter daysBacklog '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SM'!

!SMSqueakMap commentStamp: '<historical>' prior: 0!
A SqueakMap is a model for a Squeak package metacatalog - a catalog of all available Squeak packages.

One master instance will live on a server on the Internet - for example at squeakfoundation.org. Then each Squeak connected to the Internet has one instance that is synchronized incrementally against the master. This way every user can have an updated catalog of all available Squeak software on the planet. :-)

A SqueakMap essentially contains instances of SMCard which are "library cards" for Squeak packages and SMCategories which are centrally registered values that can be included as attributes in an SMCard. An example of such a category would be different kinds of licenses, topic etc. etc.

A SqueakMap knows how to store itself in a chunkformat logfile and how to load itself from such a logfile.
The methods in public-protocol are the methods used on a master SqueakMap. Those actions will get logged onto the current logfile. All methods in category logging are used when a slave SqueakMap loads incremental updates. A note: new/delete is used in the logging methods but add/remove is used in the public protocol in order to avoid name clashes.
!

HVHttpView subclass: #SMSqueakMapView
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SM'!

!SMSqueakMapView commentStamp: '<historical>' prior: 0!
A HTTP view for the SqueakMap.!


!SMModule methodsFor: 'views' stamp: 'gh 3/16/2002 22:39'!
cardViewOn: aCard
	"Wrap the card in a view."

	^SMCardView on: aCard parent: self! !

!SMModule methodsFor: 'views' stamp: 'gh 3/16/2002 22:08'!
serverViewOn: aServer
	"Wrap the server in a view.
	NOTE: We have no request to set here - it will be inserted
	in #process:. Since we are being called in multiple Processes we can't hold onto
	th request in an instvar."

	^SMServerView on: aServer parent: self! !

!SMModule methodsFor: 'views' stamp: 'gh 3/19/2002 18:57'!
squeakMapViewOn: aSqueakMap
	"Wrap the map in a view.
	NOTE: We have no request to set here - it will be inserted
	in #process:. Since we are being called in multiple Processes we can't hold onto
	th request in an instvar."

	^SMSqueakMapView on: aSqueakMap parent: self! !


!SMObject methodsFor: 'initialize-release' stamp: 'gh 3/17/2002 02:08'!
initialize
	"Initialize."

	created _ TimeStamp current asSeconds.
	updated _ created! !

!SMObject methodsFor: 'accessing' stamp: 'gh 3/16/2002 23:12'!
created
	^TimeStamp fromSeconds: created! !

!SMObject methodsFor: 'accessing' stamp: 'gh 3/16/2002 23:12'!
created: c
	created _ c! !

!SMObject methodsFor: 'accessing' stamp: 'gh 3/16/2002 23:12'!
createdAsSeconds
	^created! !

!SMObject methodsFor: 'accessing' stamp: 'gh 3/16/2002 23:12'!
id
	^id! !

!SMObject methodsFor: 'accessing' stamp: 'gh 3/17/2002 01:11'!
id: anId
	id _ anId! !

!SMObject methodsFor: 'accessing' stamp: 'gh 3/17/2002 01:44'!
updated
	^TimeStamp fromSeconds: updated! !

!SMObject methodsFor: 'accessing' stamp: 'gh 3/17/2002 01:44'!
updated: c
	updated _ c! !

!SMObject methodsFor: 'accessing' stamp: 'gh 3/17/2002 01:45'!
updatedAsSeconds
	^updated! !

!SMObject methodsFor: 'logging' stamp: 'gh 3/19/2002 22:07'!
logDeleteOn: aStream
	"Log a deletion of me on the stream."

	aStream cr; nextChunkPut: 'self deleteCardOrCategory: ', id asString storeString! !

!SMObject methodsFor: 'logging' stamp: 'gh 3/19/2002 19:54'!
logOn: aStream
	"Log me on the stream."

	aStream cr; nextChunkPut: self definition! !


!SMCard methodsFor: 'initialize-release' stamp: 'gh 3/17/2002 01:37'!
categories: anArray
	anArray do: [:i | self addCategory: (SMCategory withId: i)]! !

!SMCard methodsFor: 'initialize-release' stamp: 'gh 3/19/2002 20:52'!
initialize
	"Initialize card."

	super initialize.
	name _ currentVersion _ summary _ description _ homePageUrl _ ''.
	downloadUrl _ author _ maintainer _ registrator _ modulePath _ ''.
	categories _ OrderedCollection new! !

!SMCard methodsFor: 'logging' stamp: 'gh 3/19/2002 21:15'!
created: c updated: u name: n currentVersion: v summary: s description: d homePageUrl: h downloadUrl: du author: a maintainer: m registrator: r password: p modulePath: mp categories: cats 
	"Method used when recreating from storeOn: format."

	created _ c.
	updated _ u.
	name _ n.
	currentVersion _ v.
	summary _ s.
	description _ d.
	homePageUrl _ h.
	downloadUrl _ du.
	author _ a.
	maintainer _ m.
	registrator _ r.
	password _ p.
	modulePath _ mp.
	cats
		do: [:idString | self
				addCategory: (SMCategory withId: idString)]! !

!SMCard methodsFor: 'logging' stamp: 'gh 3/19/2002 21:42'!
definition

	^String streamContents: [:stream |
		stream nextPutAll: '(self newCardWithId: ', id asString storeString, ')'.
		stream crtab; nextPutAll: 'created: '.
		created storeOn: stream.
		stream crtab; nextPutAll: 'updated:'.
		updated storeOn: stream.
		stream crtab; nextPutAll: 'name: '.
		name storeOn: stream.
		stream crtab; nextPutAll: 'currentVersion:'.
		currentVersion storeOn: stream.
		stream crtab; nextPutAll: 'summary: '.
		summary storeOn: stream.
		stream crtab; nextPutAll: 'description:'.
		description storeOn: stream.
		stream crtab; nextPutAll: 'homePageUrl: '.
		homePageUrl storeOn: stream.
		stream crtab; nextPutAll: 'downloadUrl:'.
		downloadUrl storeOn: stream.
		stream crtab; nextPutAll: 'author: '.
		author storeOn: stream.
		stream crtab; nextPutAll: 'maintainer:'.
		maintainer storeOn: stream.
		stream crtab; nextPutAll: 'registrator:'.
		registrator storeOn: stream.
		stream crtab; nextPutAll: 'password:'.
		password storeOn: stream.
		stream crtab; nextPutAll: 'modulePath: '.
		modulePath storeOn: stream.
		stream crtab; nextPutAll: 'categories: '.
		((categories collect: [:c | c id asString]) asArray) storeOn: stream]

! !

!SMCard methodsFor: 'logging' stamp: 'gh 3/19/2002 22:50'!
logDeleteCategory: category on: aStream
	"Log a deletion of a category in me on the stream."

	aStream cr; nextChunkPut: 'self deleteCategory: ', category id asString storeString, ' inCard: ', id asString storeString! !

!SMCard methodsFor: 'logging' stamp: 'gh 3/19/2002 22:14'!
logDeleteOn: aStream
	"Log a deletion of me on the stream."

	aStream cr; nextChunkPut: 'self deleteCard: ', id asString storeString! !

!SMCard methodsFor: 'logging' stamp: 'gh 3/19/2002 22:50'!
logNewCategory: category on: aStream
	"Log an addition of a category in me on the stream."

	aStream cr; nextChunkPut: 'self newCategory: ', category id asString storeString, ' inCard: ', id asString storeString! !

!SMCard methodsFor: 'modifying' stamp: 'gh 3/19/2002 20:13'!
addCategory: aCategory
	^categories add: aCategory! !

!SMCard methodsFor: 'modifying' stamp: 'gh 3/19/2002 22:43'!
removeCategory: aCategory
	^categories remove: aCategory! !

!SMCard methodsFor: 'accessing' stamp: 'gh 3/19/2002 20:11'!
author
	^author! !

!SMCard methodsFor: 'accessing' stamp: 'gh 3/17/2002 01:24'!
author: aString
	author _ aString! !

!SMCard methodsFor: 'accessing' stamp: 'gh 3/19/2002 22:01'!
categories
	^categories! !

!SMCard methodsFor: 'accessing' stamp: 'gh 3/19/2002 20:11'!
currentVersion
	^currentVersion! !

!SMCard methodsFor: 'accessing' stamp: 'gh 3/17/2002 01:22'!
currentVersion: aString
	currentVersion _ aString! !

!SMCard methodsFor: 'accessing' stamp: 'gh 3/19/2002 20:11'!
description
	^description! !

!SMCard methodsFor: 'accessing' stamp: 'gh 3/17/2002 01:23'!
description: aString
	description _ aString! !

!SMCard methodsFor: 'accessing' stamp: 'gh 3/19/2002 20:12'!
downloadUrl
	^downloadUrl! !

!SMCard methodsFor: 'accessing' stamp: 'gh 3/17/2002 01:24'!
downloadUrl: aString
	downloadUrl _ aString! !

!SMCard methodsFor: 'accessing' stamp: 'gh 3/19/2002 20:12'!
homePageUrl
	^homePageUrl! !

!SMCard methodsFor: 'accessing' stamp: 'gh 3/17/2002 01:24'!
homePageUrl: aString
	homePageUrl _ aString! !

!SMCard methodsFor: 'accessing' stamp: 'gh 3/19/2002 20:12'!
maintainer
	^maintainer! !

!SMCard methodsFor: 'accessing' stamp: 'gh 3/17/2002 01:24'!
maintainer: aString
	maintainer _ aString! !

!SMCard methodsFor: 'accessing' stamp: 'gh 3/19/2002 20:12'!
modulePath
	^modulePath! !

!SMCard methodsFor: 'accessing' stamp: 'gh 3/17/2002 01:25'!
modulePath: aString
	modulePath _ aString! !

!SMCard methodsFor: 'accessing' stamp: 'gh 3/19/2002 20:12'!
name
	^name! !

!SMCard methodsFor: 'accessing' stamp: 'gh 3/17/2002 01:22'!
name: aName
	name _ aName! !

!SMCard methodsFor: 'accessing' stamp: 'gh 3/19/2002 21:14'!
password
	^password! !

!SMCard methodsFor: 'accessing' stamp: 'gh 3/19/2002 21:24'!
password: aString
	password _ aString! !

!SMCard methodsFor: 'accessing' stamp: 'gh 3/19/2002 20:53'!
registrator
	^registrator! !

!SMCard methodsFor: 'accessing' stamp: 'gh 3/19/2002 20:54'!
registrator: aString
	registrator _ aString! !

!SMCard methodsFor: 'accessing' stamp: 'gh 3/19/2002 20:12'!
summary
	^summary! !

!SMCard methodsFor: 'accessing' stamp: 'gh 3/17/2002 01:23'!
summary: aString
	summary _ aString! !


!SMCategory methodsFor: 'accessing' stamp: 'gh 3/17/2002 02:09'!
initialize
	super initialize.
	name _ summary _ url _ ''.
	mandatory _ false! !

!SMCategory methodsFor: 'accessing' stamp: 'gh 3/17/2002 01:48'!
mandatory: aBoolean
	mandatory _ aBoolean! !

!SMCategory methodsFor: 'accessing' stamp: 'gh 3/17/2002 01:47'!
name: aString
	name _ aString! !

!SMCategory methodsFor: 'accessing' stamp: 'gh 3/17/2002 01:47'!
summary: aString
	summary _ aString! !

!SMCategory methodsFor: 'accessing' stamp: 'gh 3/17/2002 01:48'!
url: aString
	url _ aString! !

!SMCategory methodsFor: 'logging' stamp: 'gh 3/19/2002 20:04'!
created: c updated: u name: n summary: s url: uu mandatory: m
	"Method used when recreating from storeOn: format."

	created _ c.
	updated _ u.
	name _ n.
	summary _ s.
	url _ uu.
	mandatory _ m! !

!SMCategory methodsFor: 'logging' stamp: 'gh 3/19/2002 21:43'!
definition
	^String streamContents: [:stream |
		stream nextPutAll: '(self newCategoryWithId: ', id asString storeString, ')'.
		stream crtab; nextPutAll: 'created: '.
		created storeOn: stream.
		stream crtab; nextPutAll: 'updated:'.
		updated storeOn: stream.
		stream crtab; nextPutAll: 'name: '.
		name storeOn: stream.	
		stream crtab; nextPutAll: 'summary: '.
		summary storeOn: stream.
		stream crtab; nextPutAll: 'url: '.
		url storeOn: stream.
		stream crtab; nextPutAll: 'mandatory: '.
		mandatory storeOn: stream]! !

!SMCategory methodsFor: 'logging' stamp: 'gh 3/19/2002 22:14'!
logDeleteOn: aStream
	"Log a deletion of me on the stream."

	aStream cr; nextChunkPut: 'self deleteCategory: ', id asString storeString! !


!SMObject class methodsFor: 'instance creation' stamp: 'gh 3/19/2002 18:50'!
newWithId: anUUIDString
	"Create a new object with a given UUID as a String.
	This method is used when we create instances from a logfile etc."

	| obj |
	obj _ super new initialize; id: (UUID fromString: anUUIDString).
	^Ids at: obj id put: obj
	! !

!SMObject class methodsFor: 'instance creation' stamp: 'gh 3/19/2002 18:51'!
withId: anUUIDString
	"Lookup an already created instance with that UUID."

	^Ids at: (UUID fromString: anUUIDString) ifAbsent: [nil]! !

!SMObject class methodsFor: 'class initialization' stamp: 'gh 3/17/2002 02:02'!
initialize
	"self initialize"

	Ids _ Dictionary new! !

!SMObject class methodsFor: 'class initialization' stamp: 'gh 3/19/2002 18:34'!
new
	"Create a new object with an UUID to ensure unique identity.
	We also register all instances in a Dictionary."

	| obj |
	obj _ super new initialize.
	obj id: UUID new.
	^Ids at: obj id put: obj! !


!SMSqueakMap methodsFor: 'accessing' stamp: 'gh 3/19/2002 22:22'!
cards
	^cards! !

!SMSqueakMap methodsFor: 'accessing' stamp: 'gh 3/19/2002 22:22'!
categories
	^categories! !

!SMSqueakMap methodsFor: 'accessing' stamp: 'gh 3/19/2002 23:06'!
daysBacklog
	^daysBacklog! !

!SMSqueakMap methodsFor: 'accessing' stamp: 'gh 3/19/2002 23:06'!
daysBacklog: days
	daysBacklog _ days! !

!SMSqueakMap methodsFor: 'logging' stamp: 'gh 3/19/2002 19:52'!
createNewLog
	"Create a new logfile. First chunk is current date."

	| file |
	[file _ dir newFileNamed: (dir nextNameFor: 'card' extension: 'log').
	file nextChunkPut: Date today storeString]
		ensure: [file close]! !

!SMSqueakMap methodsFor: 'logging' stamp: 'gh 3/19/2002 22:15'!
deleteCard: anIdString
	"Delete a card or category."

	^cards removeKey: (UUID fromString: anIdString)
! !

!SMSqueakMap methodsFor: 'logging' stamp: 'gh 3/19/2002 22:40'!
deleteCategory: anIdString
	"Delete a category. We also delete all
	references to it from any cards."

	| cat |
	cat _ categories at: (UUID fromString: anIdString).
	cards do: [:card | card categories remove: cat ifAbsent:[nil] ].
	^categories removeKey: cat id
! !

!SMSqueakMap methodsFor: 'logging' stamp: 'gh 3/19/2002 22:47'!
deleteCategory: catIdString inCard: cardIdString
	"Delete a category from a card."

	| cat card |
	cat _ categories at: (UUID fromString: catIdString).
	card _ cards at: (UUID fromString: cardIdString).
	^card removeCategory: cat
! !

!SMSqueakMap methodsFor: 'logging' stamp: 'gh 3/19/2002 19:30'!
loadDateFrom: aStream
	"Load the creation date from the first chunk."

	^Compiler evaluate: aStream nextChunk
! !

!SMSqueakMap methodsFor: 'logging' stamp: 'gh 3/19/2002 21:57'!
loadUpdatesFrom: aStream log: doLogging
	"Load updates (cards or categories) from the stream.
	This updates the model in memory and if <doLogging>
	is true we also log the updates in the local log file."

	| chunk file |
	[ doLogging ifTrue:[ file _ self openLogFile setToEnd ].
	[aStream atEnd]
		whileFalse: [
			chunk _ aStream nextChunk.
			Compiler evaluate: chunk for: self logged: false.
			doLogging ifTrue: [file cr; nextChunkPut: chunk].
		]] ensure: [
			aStream close.
			doLogging ifTrue:[file close]]! !

!SMSqueakMap methodsFor: 'logging' stamp: 'gh 3/19/2002 23:14'!
log: aCardOrCategory
	"Pick the newest logfile available and log the object in it."
	
	| file |
	[ file _ self openLogFile setToEnd.
	aCardOrCategory logOn: file.
	self logIncrementedTransactionCounterOn: file ]
		ensure: [file close]
! !

!SMSqueakMap methodsFor: 'logging' stamp: 'gh 3/19/2002 23:14'!
logDelete: aCardOrCategory
	"Pick the newest logfile available and log the deletion in it."
	
	| file |
	[ file _ self openLogFile setToEnd.
	aCardOrCategory logDeleteOn: file.
	self logIncrementedTransactionCounterOn: file ]
		ensure: [file close]
! !

!SMSqueakMap methodsFor: 'logging' stamp: 'gh 3/19/2002 23:15'!
logIncrementedTransactionCounterOn: aStream
	"Increment the transaction counter and
	log the current number."

	transactionCounter _ transactionCounter + 1.
	self logTransactionCounterOn: aStream
! !

!SMSqueakMap methodsFor: 'logging' stamp: 'gh 3/19/2002 23:16'!
logTransactionCounterOn: aStream
	"Log the current transaction number."

	aStream cr; nextChunkPut: 'self transactionCounter: ', transactionCounter storeString
! !

!SMSqueakMap methodsFor: 'logging' stamp: 'gh 3/19/2002 22:25'!
newCardWithId: anIdString
	"Create a new card and add it to me."

	| card |
	card _ SMCard newWithId: anIdString.
	^cards at: card id put: card
! !

!SMSqueakMap methodsFor: 'logging' stamp: 'gh 3/19/2002 22:52'!
newCategory: catIdString inCard: cardIdString
	"Add a category to a card."

	| cat card |
	cat _ categories at: (UUID fromString: catIdString).
	card _ cards at: (UUID fromString: cardIdString).
	^card addCategory: cat
! !

!SMSqueakMap methodsFor: 'logging' stamp: 'gh 3/19/2002 22:24'!
newCategoryWithId: anIdString
	"Create a new category and add it to me."

	| cat |
	cat _ SMCategory newWithId: anIdString.
	^categories at: cat id put: cat
! !

!SMSqueakMap methodsFor: 'logging' stamp: 'gh 3/19/2002 19:49'!
openLogFile
	"Pick the newest logfile available and open it."

	| fileName |
	fileName _ dir lastNameFor: 'card' extension: 'log'.
	fileName ifNil: [^nil].
	^dir oldFileNamed: fileName
! !

!SMSqueakMap methodsFor: 'logging' stamp: 'gh 3/19/2002 23:40'!
openLogFileReadOnly
	"Pick the newest logfile available and open it."

	| fileName |
	fileName _ dir lastNameFor: 'card' extension: 'log'.
	fileName ifNil: [^nil].
	^dir readOnlyFileNamed: fileName
! !

!SMSqueakMap methodsFor: 'logging' stamp: 'gh 3/19/2002 23:08'!
transactionCounter: aCount
	"Set the last known transaction number."

	transactionCounter _ aCount
! !

!SMSqueakMap methodsFor: 'initialize-release' stamp: 'gh 3/19/2002 23:05'!
initialize
	"Create the local directory for the SqueakMap.
	We also try to load the map from the log
	if there is one present."

	dir _ FileDirectory default.
	(dir directoryExists: 'sm')
		ifFalse:[dir createDirectory: 'sm'].
	dir _ dir directoryNamed: 'sm'.
	transactionCounter _ 0.
	daysBacklog _ 182. "about 6 months"
	self reloadLog! !

!SMSqueakMap methodsFor: 'initialize-release' stamp: 'gh 3/19/2002 23:06'!
loadLog
	"Pick the current logfile - if there is none, create a new and return.
	Otherwise we load the creation date (stored in the beginning).
	Then load all cards in it and build the model. If the creation date is older than 6 months
	we recreate a new logfile in a condensed form and rotate to that."
	
	| file creationDate |
	[ file _ self openLogFile.
	file ifNil: [^self createNewLog].
	creationDate _ self loadDateFrom: file.
	self loadUpdatesFrom: file log: false. "we don't log since we are reading from it!!"
	(creationDate < (Date today subtractDays: daysBacklog))
		ifTrue:[self saveNewLog]] ensure: [file ifNotNil: [file close]]! !

!SMSqueakMap methodsFor: 'views' stamp: 'gh 3/19/2002 18:56'!
viewFor: uiObject
	"This is a double dispatch mechanism for multiple views
	for multiple uis."

	^uiObject squeakMapViewOn: self! !

!SMSqueakMap methodsFor: 'public-procotol' stamp: 'gh 3/19/2002 22:18'!
addCard: aCard
	"Add or update a new card.
	Log it in the logfile and register it in the cards."

	self log: aCard.
	^cards at: aCard id put: aCard! !

!SMSqueakMap methodsFor: 'public-procotol' stamp: 'gh 3/19/2002 22:19'!
addCategory: aCategory
	"Add or update a new category.
	Log it in the logfile and register it in the categories."

	self log: aCategory.
	^categories at: aCategory id put: aCategory! !

!SMSqueakMap methodsFor: 'public-procotol' stamp: 'gh 3/19/2002 23:14'!
addCategory: category inCard: card
	"Add a category in a card.
	Log the removal."

	| file |
	[ file _ self openLogFile setToEnd.
	card logNewCategory: category on: file.
	self logIncrementedTransactionCounterOn: file ]
		ensure: [file close].
	^card addCategory: category
! !

!SMSqueakMap methodsFor: 'public-procotol' stamp: 'gh 3/19/2002 23:02'!
reloadLog
	"First we clear the current content and then we load the log.
	This should not lead to any differences!!"
	
	categories _ Dictionary new.
	cards _ Dictionary new.
	self loadLog! !

!SMSqueakMap methodsFor: 'public-procotol' stamp: 'gh 3/19/2002 22:16'!
removeCard: aCard
	"Remove a card.
	Log it in the logfile and remove it from the cards."
	
	self logDelete: aCard.
	^cards removeKey: aCard id! !

!SMSqueakMap methodsFor: 'public-procotol' stamp: 'gh 3/19/2002 22:37'!
removeCategory: aCategory
	"Remove a category. Remove the category from all
	cards referencing it. Note that we don't need to log
	all those changed cards since the updating logic in the
	slave SqueakMaps will also remove the category from any
	cards referencing it.
	Log it in the logfile and remove it from the categories."

	cards do: [:card | card categories remove: aCategory ifAbsent:[nil]].
	self logDelete: aCategory.
	^categories removeKey: aCategory id! !

!SMSqueakMap methodsFor: 'public-procotol' stamp: 'gh 3/19/2002 23:14'!
removeCategory: category inCard: card
	"Remove a category from a card.
	Log the removal."

	| file |
	[ file _ self openLogFile setToEnd.
	card logDeleteCategory: category on: file.
	self logIncrementedTransactionCounterOn: file ]
		ensure: [file close].
	^card removeCategory: category
! !

!SMSqueakMap methodsFor: 'public-procotol' stamp: 'gh 3/19/2002 23:10'!
saveNewLog
	"Create a new logfile and store the model condensed in it.
	A slave SqueakMap can do this at any time, it is done automatically at least
	once every 6 months. A master SqueakMap should not do this often at all -
	the more seldom it does the longer backlog it has and can feed slaves with incremental
	updates instead of forcing them to a complete download.
	First chunk is current date. Categories are saved first, then all the cards.
	At the end we tack on the last known transactionId."

	| file |
	self createNewLog.
	[ file _ self openLogFile setToEnd.
	categories valuesDo: [:cat | cat logOn: file].
	cards valuesDo: [:card | card logOn: file].
	self logTransactionCounterOn: file]
		ensure: [file close]! !

!SMSqueakMap methodsFor: 'public-procotol' stamp: 'gh 3/20/2002 00:34'!
updatesSinceTransaction: lastTransaction
	"Produce a String with all transactions from the logfile
	since <lastTransaction>. We go to the end of the file,
	then we search backwards for the text 'transactionCounter: xxx'.
	The implementation uses the new method #findStringFromEnd: which
	should be reasonably fast. If we find no such marker it means that
	this logfile has been condensed, then we return nil. The client
	will have to ask for the whole file instead."

	| file found result key |
	[ file _ self openLogFileReadOnly setToEnd.
	key _ 'self transactionCounter: ', lastTransaction storeString, '!!'.
	found _ file findStringFromEnd: key.
	found = 0 ifTrue:[result _ nil] ifFalse: [ file skip: key size. result _ file upToEnd ]]
		ensure: [file close].
	^result
! !


!SMSqueakMap class methodsFor: 'instance creation' stamp: 'gh 3/16/2002 22:04'!
new
	"Create a new server."

	^super new initialize! !


!SMSqueakMapView methodsFor: 'initialize-release' stamp: 'gh 3/16/2002 22:40'!
initialize
	title _ 'SqueakMap'.
	prefix _ ''.
! !

!SMSqueakMapView methodsFor: 'views' stamp: 'gh 3/16/2002 22:40'!
cardViewOn: aCard
	"Wrap the card in a view."

	^SMCardView on: aCard parent: self! !

!SMSqueakMapView methodsFor: 'old' stamp: 'gh 3/16/2002 22:07'!
account
	"This is the users account page.
	We verify that we are logged in and delegate the rest
	onto the correct view for an account based on the rest of the urlpath."

	| account |
	(account _ self currentAccount) ifNil: [^self notloggedin ].
	^(account viewFor: self) dispatch! !

!SMSqueakMapView methodsFor: 'old' stamp: 'gh 3/16/2002 22:07'!
accounts
	"Lists all accounts."

	^self builder start; h1: 'All accounts'; br;
		table: model accounts
		heads: #(Username Name 'Logged in' 'Last access (if logged in)' 'Last logout')
		block: [:a | {a username.
				(self link: 'mailto:', a info email text: a info fullName).
				(a isLoggedIn ifTrue: ['YES'] ifFalse:['NO']).
				(a lastAccessed) printString.
				(a lastLogout) printString}];
		linkBack: 'Back'; hr; srcLink; end; yourself! !

!SMSqueakMapView methodsFor: 'old' stamp: 'gh 3/20/2002 00:39'!
default
	"Default request for the top view. Show the first page."

	| b search |
	b _ self builder.
	b start; h1: 'SqueakMap';
		p: 'Welcome to the SqueakMap!!';
		html: '<p>SqueakMap is a catalogue over hopefully <b>all code ever written</b> in ';
		link: 'http://www.squeak.org' text: 'Squeak'; html: ':';
		ul: #('Anyone can register a "library card" describing a Squeak package.'
			'The cards are easily searchable and browseable in categories.'
			'You can easily create your own cached local catalog in Squeak.');
		html: '<p>Sofar <b>', model cards size fullPrintString, '</b> Squeak packages have been registered, click '; linkLocal: '/newcard' text: 'here'; html: ' to register <i>yours</i>.</p>'.
	b html: '<p>If you want to add/edit/remove categories - click '; linkLocal: '/editcategories' text: 'here'; html: '</p>.'.
	b hr; h1: 'Search'; postForm.
	search _ b html: 'Words: '; inputText.
	b endForm.
	b end.
	^b! !

!SMSqueakMapView methodsFor: 'old' stamp: 'gh 3/16/2002 22:07'!
login
	"Handle a login."

	^self login: 'Login to SqueakDot'! !

!SMSqueakMapView methodsFor: 'old' stamp: 'gh 3/16/2002 22:07'!
login: aHeading
	"Handle a login."

	^self login: aHeading url: nil! !

!SMSqueakMapView methodsFor: 'old' stamp: 'gh 3/16/2002 22:07'!
login: aHeading url: url
	"Handle a login."

	| username password account requestedUrl b |
	b _ self builder.
	b start; h1: aHeading; postForm.
	username _ (b html: 'Username: '; inputText) value.
	password _ (b br; html: 'Password: '; inputPassword) value.
	requestedUrl _ b inputHidden: #requestedUrl.
	b br; submit: 'Login'; endForm; linkLocal: '/newaccount' text: 'New account'.
	b br; linkBack: 'Back'; hr; br; srcLink; end.
	b ifPost: [
		(requestedUrl value = (prefix, '/login')) ifTrue:[requestedUrl value: self prefix].
		account _ model accountForUsername: username.
		account ifNil: [^self message: 'No user with that name' linklocal: 'login'].
		(account verifyPassword: password)
			ifTrue:[
				req session attributes at: #account put: account.
				^req redirectTo: requestedUrl value ]
			ifFalse:[
				req session attributes at: #failedAccount put: account.
				^self wrongpassword ]
	].
	requestedUrl value: (url ifNil: [req url] ifNotNil: [url]).
	^b
! !

!SMSqueakMapView methodsFor: 'old' stamp: 'gh 3/16/2002 22:07'!
logout
	"Handle a logout. If it fails we are probably not logged in.
	This version goes to the top page. A variant would be to go to the login page:
	^self login: request heading: 'Logged out' url: self prefix, '/account'"

	super logout ifFalse: [^self notloggedin].
	^req redirectTo: self prefix! !

!SMSqueakMapView methodsFor: 'old' stamp: 'gh 3/19/2002 22:19'!
newcard
	"Register a new card."

	| msg b name currentVersion summary description homePageUrl downloadUrl modulePath author maintainer registrator card password |
	b _ self builder.
	b start; h1: 'New SqueakMap card'; postForm.
	b p: 'Fields in <b>bold</b> are mandatory. No syntactical checks are made so <b>be careful</b>.'.
	name _ (b html: '<b>Name (a cool name of course):</b> '; br; inputTextSize: 40) value.
	currentVersion _ (b br; html: '<b>Version (ex: "1.22.99-beta-prepatch7"):</b> ';br; inputText) value.
	summary _ (b br; html: '<b>Summary (1 line):</b> '; br; inputTextSize: 80) value.
	description _ (b br; html: 'Description (5-10 lines): ';br; textAreaRows: 10 cols: 80) value.
	homePageUrl _ (b br; html: 'Homepage (url ex: "http://www.here.com/killerapp/index.html"): '; br; inputTextSize: 40) value.
	downloadUrl _ (b br; html: 'Download (url ex: "http://www.somewhere.com/killerapp.cs.gz"): '; br; inputTextSize: 40) value.
	modulePath _ (b br; html: 'ModulePath (ex: "People gh Killerapp"): '; br; inputTextSize: 40) value.
	author _ (b br; html: '<b>Author (The original author, ex: "Joe Schmoe &lt;joe@schmoe.com&gt;"):</b> '; br; inputTextSize: 40) value.
	maintainer _ (b br; html: '<b>Maintainer (The person to contact with any questions):</b> '; br; inputTextSize: 40) value.
	registrator _ (b br; html: '<b>Registrator (The person typing in this card, thats you!!):</b> '; br; inputTextSize: 40) value.
	password _ (b br; html: '<b>Password (you will need it in order to update the card later):</b> '; br; inputText) value.
	b br; submit: 'Create'; reset: 'Reset'; endForm.
	b h2: [ msg _ b var ].
	b linkBack: 'Back'; hr; br; end.
	b ifPost: [
		name isEmpty ifTrue: [msg value: 'Name is mandatory'. ^b].
		currentVersion isEmpty ifTrue: [msg value: 'Version is mandatory'. ^b].
		summary isEmpty ifTrue: [msg value: 'Summary is mandatory'. ^b].
		author isEmpty ifTrue: [msg value: 'Author is mandatory'. ^b].
		maintainer isEmpty ifTrue: [msg value: 'Maintainer is mandatory'. ^b].
		registrator isEmpty ifTrue: [msg value: 'Registrator is mandatory'. ^b].
		password isEmpty ifTrue: [msg value: 'Password is mandatory'. ^b].
		[ card _ (SMCard new)
			name: name; currentVersion: currentVersion; summary: summary;
			description: description; homePageUrl: homePageUrl; downloadUrl: downloadUrl;
			modulePath: modulePath; author: author; maintainer: maintainer; registrator: registrator;
			password: password.
		model addCard: card ]
			ifError: [msg value: 'Something went wrong when creating card'. ^b].
		^self message: 'Card created successfully' linklocal: ''].
	^b! !

!SMSqueakMapView methodsFor: 'old' stamp: 'gh 3/16/2002 22:07'!
notloggedin
	"Not logged in."

	^self login: 'Not logged in'! !

!SMSqueakMapView methodsFor: 'old' stamp: 'gh 3/16/2002 22:07'!
public
	"This is a request for a private resource in an account."

	| account method view username |
	username _ self methodAfter: prefix, '/public'.
	method _ self methodAfter: prefix, '/public/', username.
	account _ model accountForUsername: username.
	account ifNil: [^self message: 'Unknown username' ].
	view _ SDPublicAccountView on: account parent: self. "should use viewFor: with double dispatch... how?"
	^view dispatchOn: method! !

!SMSqueakMapView methodsFor: 'old' stamp: 'gh 3/20/2002 00:39'!
updates
	"The user requests a compressed logfile for all transactions since
	the given transaction number."

	^model updatesSinceTransaction: (req getFields at: #transaction) asNumber! !

!SMSqueakMapView methodsFor: 'old' stamp: 'gh 3/20/2002 00:47'!
updatesgz
	"The user requests a compressed logfile for all transactions since
	the given transaction number."

	| buffer |
	buffer _ RWBinaryOrTextStream on: String new.
	(GZipWriteStream on: buffer) nextPutAll: self updates; close.
	^buffer contents! !

!SMSqueakMapView methodsFor: 'old' stamp: 'gh 3/16/2002 22:07'!
wrongpassword
	"The user wrote the wrong password.
	First we make sure that it is 'our' page that was posted."

	| phraseword account |
	account _ req session attributes at: #failedAccount ifAbsent: [^self serverError: 'No account found'].
	(req url endsWith: '/wrongpassword')
		ifTrue:[
			req isPostRequest ifTrue:[
				phraseword _ req postFields at: 'phraseword' ifAbsent: [].
				(account verifyPhraseword: phraseword)
					ifTrue:[
						req session attributes removeKey: #failedAccount.
						req session attributes at: #account put: account.
						^req redirectTo: self prefix ]
					ifFalse:[ "fall through and resend the form" ]
			]
	].
^'<html>
<head><title>SqueakDot</title></head>
<body>
<h1>Wrong password</h1>
<form action="', prefix, '/wrongpassword" method="post">
Username: ', account username, '<br>
<input type="hidden" name="username" value="', account username, '">
Your passphrase is: <b>', account passphrase, '</b><br>
Phraseword: <input type="password" name="phraseword"><br>
<input type="submit" value="Login">
</form>
<a href="', prefix, '/login">Try to login normally again</a>
</body>
</html>'
! !

SMObject initialize!
