'From Squeak3.1beta of 3 October 2001 [latest update: #4418] on 17 October 2001 at 2:34:26 pm'!
"Change Set:		spsConfigFile
Date:			12 October 2001
Author:			Steven Swerling

This changeset gives you a class called ConfigFile that acts as a wrapper for accessing a file that is in the format of a Windows ini file ([Topics] with key=value pairs). The intent is to make it very easy to get/set values in a config file. All of the values in the file are cached when the ConfigFile is created, so accessing values is as fast as a Dictionary access (after the initial read of the file, which itself is pretty fast). (When values are set, they are set both in the cache and in the file itself, of course). The class comment of ConfigFile gives a thorough description."!

Object subclass: #ConfigFile
	instanceVariableNames: 'directory localFileName topicDict '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'sps-utility'!

!ConfigFile commentStamp: '<historical>' prior: 0!
This class maintains a configuration file in a format similar to the Windows ini file. A configuration file is composed of "topics", and the topics are key/value pairs. Following is an example:

[Default Colors]
text cursor=255 255 255
highlight=255 0 255
[General]
skill=novice
speed=slow

In this example, there are 2 topics, "Default Colors" and "General". The General topic contains a key called "skill" with the value "novice", and a key called "speed" with value "slow". Topics must begin a new line and be surrounded by square brackets. Key/value pairs must be separated by an equal sign (=), so the key cannot have an equal sign. The key is terminated by the equal sign, the value is terminated by the end of the line (CR, LF, or CRLF).

See the accessing protocol for setting and getting values from the config file.

===== Here are some example usages=========

"(doit to the following code)"
MyConfigFile _ ConfigFile 
	fileName: 'test.ini'
	directory: FileDirectory default
	createIfNotThere: true.

MyConfigFile setKey: 'myColor' topic: 'Test Colors' value: Color orange .
MyConfigFile setKey: 'myColor' topic: 'More Test Colors' value: Color blue .
MyConfigFile setKey: 'myColor2' topic: 'Test Colors' value: Color green .
MyConfigFile setKey: 'myBool' topic: 'Test Bool' value: true .

"Print it on the following lines"
MyConfigFile getColor: 'myColor' topic: 'Test Colors' default: nil. 
MyConfigFile getColor: 'myColor2' topic: 'Test Colors' default: nil. 
MyConfigFile getColor: 'myColor' topic: 'More Test Colors' default: nil. 
MyConfigFile getBoolean: 'myBool' topic: 'Test Bool' default: false. 

"Tweak the values in test.ini by hand using a Filelist, 
save the file, then doit to the following method"
MyConfigFile reread. "After executing this method, do the above 4 lines of code again"
=======

Structure:
 directory 		FileDirectory		The directory containing the config file
 localFileName	String				The name of the config file
 topicDict			Dictionary			Cache of values


The topicDict contains a Dictionary where the keys are topic names, the values are themselves a Dictionary, the key/value dictionary for that topic. 

When setKey is called, the key is written both in the topicDict and in the file itself.

The file is not accessed during the getKey operations, only the topicDict cache, which is read in completely during initialization of the ConfigFile, or if a call to #reread is made.!

ConfigFile class
	instanceVariableNames: ''!

!ConfigFile methodsFor: 'accessing' stamp: 'sps 10/15/2001 18:32'!
allTopics
	^self topicDict keys! !

!ConfigFile methodsFor: 'accessing' stamp: 'sps 9/26/2001 23:09'!
directory
	^directory! !

!ConfigFile methodsFor: 'accessing' stamp: 'sps 9/26/2001 23:09'!
directory: aFileDirectory
	^directory _ aFileDirectory! !

!ConfigFile methodsFor: 'accessing'!
getBoolean: aKeyString topic: aTopicString
	"Get the value, or if not found, return default (use empty string as default)"

	^self getBoolean: aKeyString topic: aTopicString default: true.! !

!ConfigFile methodsFor: 'accessing' stamp: 'sps 9/28/2001 15:37'!
getBoolean: aKeyString topic: aTopicString default: aDefaultBool
	"Return value in the form of a boolean, or if not found, return default"

	| topic bool |

	(topic _ self topicDict at: aTopicString ifAbsent: [ nil ]) isNil 
		ifTrue: [ ^aDefaultBool].

 	(bool _ topic at: aKeyString ifAbsent: [ nil ]) isNil 
		ifTrue: [ ^aDefaultBool ].

	bool isString ifFalse: [ ^bool ].

	(bool asUppercase = 'TRUE') 
		ifTrue: [ bool _ true ]
		ifFalse: [
			(bool asUppercase = 'FALSE') 
				ifTrue: [ bool _ false ]
				ifFalse: [ bool _ aDefaultBool ]
 		].

	topic at: aKeyString put: bool.

	^bool! !

!ConfigFile methodsFor: 'accessing'!
getColor: aKeyString topic: aTopicString
	"Get the value, or if not found, return default (use empty string as default)"

	^self getColor: aKeyString topic: aTopicString default: nil.! !

!ConfigFile methodsFor: 'accessing' stamp: 'sps 9/28/2001 15:37'!
getColor: aKeyString topic: aTopicString default: aDefaultColor
	"Get the value, or if not found, return default"

	| topic   color scalingFactor r g b |

	(topic _ self topicDict at: aTopicString ifAbsent: [ nil ]) isNil 
		ifTrue: [ ^aDefaultColor ].

 	(color _ topic at: aKeyString ifAbsent: [ nil ]) isNil 
		ifTrue: [ ^aDefaultColor ].

	color isString ifFalse: [ ^color ].
	color _ color substrings.
	color size == 3 ifFalse: [
		"If the color was illegal, don't search for it in the future"
		topic at: aKeyString put: nil.
	].
	scalingFactor _ 255.
	r _ (color at: 1) asInteger / scalingFactor.
	g _ (color at: 2) asInteger / scalingFactor.
	b _ (color at: 3) asInteger / scalingFactor.
	topic at: aKeyString put: (color _ Color r: r g: g b: b).

	^color! !

!ConfigFile methodsFor: 'accessing'!
getKey: aKeyString topic: aTopicString
	"Get the value, or if not found, return default (use empty string as default)"

	^self getKey: aKeyString topic: aTopicString default: ''.! !

!ConfigFile methodsFor: 'accessing' stamp: 'sps 9/28/2001 15:37'!
getKey: aKeyString topic: aTopicString default: aDefaultString
	"Get the value, or if not found, return default"

	| topic value |

	(topic _ self topicDict at: aTopicString ifAbsent: [ nil ]) isNil 
		ifTrue: [ ^aDefaultString ].

 	(value _ topic at: aKeyString ifAbsent: [ nil ]) isNil 
		ifTrue: [ ^aDefaultString ].

	value isString
		ifTrue:[^value withBlanksTrimmed]
		ifFalse:[ ^value]! !

!ConfigFile methodsFor: 'accessing' stamp: 'sps 9/28/2001 15:37'!
getKeysForTopic: aTopic

	| dict |
	(dict _ self topicDict at: aTopic ifAbsent: [nil]) isNil ifTrue: [
		^OrderedCollection new
	].

	^dict keys asOrderedCollection! !

!ConfigFile methodsFor: 'accessing' stamp: 'sps 9/26/2001 23:09'!
localFileName
	^localFileName! !

!ConfigFile methodsFor: 'accessing' stamp: 'sps 9/26/2001 23:09'!
localFileName: aString
	^localFileName _ aString! !

!ConfigFile methodsFor: 'accessing' stamp: 'sps 9/28/2001 15:38'!
removeKey: aKey topic: aTopic
"Remove a key for a given topic"

	| topic |
	(topic _ self topicDict at: aTopic ifAbsent: []) isNil ifTrue: [ ^self ].
	topic removeKey: aKey ifAbsent: [].
	self saveOutTopicDict.! !

!ConfigFile methodsFor: 'accessing'!
removeTopic: aTopic
"Remove an entire topic and all of its keys"

	self topicDict removeKey: aTopic ifAbsent: [ ^self ].
	self saveOutTopicDict.! !

!ConfigFile methodsFor: 'accessing' stamp: 'sps 9/28/2001 15:44'!
setKey: aKeyString topic: aTopicString value: val
"Set the value; if topic not found, add it. If key not found, add it. Update the file if needed"
	| aValueString |

	aValueString _ self configFileForm: val.

	"See if the value is already aValueString"
	(self getKey: aKeyString topic: aTopicString) = aValueString 
		ifTrue: [ ^true ].

	^self privateSetKey: aKeyString topic: aTopicString value: aValueString! !

!ConfigFile methodsFor: 'private' stamp: 'sps 10/5/2001 17:57'!
addKey: aKeyString value: aValueString onStream: aFileStream
"Add a key to the topic -- assumes aFileStream has already been positioned after the topic string"

	|   tempStream  |

	"Create the output string"
	(tempStream _ WriteStream on: String new)
		cr;
		nextPutAll: aKeyString; 
		nextPut: $=;
		nextPutAll: (self configFileForm: aValueString); 
		cr.
	tempStream close.

	aFileStream
		spliceString: tempStream contents
		from: aFileStream position
		to: aFileStream position.! !

!ConfigFile methodsFor: 'private'!
addTopic: aTopicString onStream: aFileStream
		"Add the new topic to the end of the file"

	aFileStream 
		position: aFileStream size;
		cr;
		nextPut: $[;
		nextPutAll: aTopicString;
		nextPut: $];
		cr.! !

!ConfigFile methodsFor: 'private' stamp: 'sps 10/5/2001 17:57'!
cacheSetKey: aKeyString topic: aTopicString value: aValueString
"Update the cache w/ the new value -- called after the file is updated"	

	| dict |

	(dict _ self topicDict at: aTopicString ifAbsent: [ nil ]) ifNil: [ 
		self topicDict at: aTopicString put: (dict _ Dictionary new)	
	].

	dict at: aKeyString put: aValueString.! !

!ConfigFile methodsFor: 'private' stamp: 'sps 9/28/2001 15:35'!
configFileForm: anObject

	| stream |
	anObject isString ifTrue: [ ^anObject ].

	anObject isInteger ifTrue: [ ^anObject asString ].

	anObject isColor ifTrue: [
		stream _ WriteStream on: (String new: 50).
		stream 
			nextPutAll: (anObject red * 255) rounded asString; space;
			nextPutAll: (anObject green * 255) rounded asString; space;
			nextPutAll: (anObject blue * 255) rounded asString.
		stream close.
		^stream contents
	].

	^anObject printString! !

!ConfigFile methodsFor: 'private' stamp: 'sps 9/28/2001 13:43'!
fileStream

	^self directory fileNamed: self localFileName! !

!ConfigFile methodsFor: 'private' stamp: 'sps 9/28/2001 15:35'!
findKeyValuePair: aKeyString onStream: aFileStream
"Return the key/value Association on aFileStream where the key is aKeyString, or nil."

	|  keyAssoc |

	(keyAssoc _ self getNextKeyValue: aFileStream) 
		ifNil: [ ^nil ].

	(keyAssoc key = aKeyString) 
		ifTrue: [ ^keyAssoc ].

	^self findKeyValuePair: aKeyString onStream: aFileStream! !

!ConfigFile methodsFor: 'private' stamp: 'sps 10/5/2001 18:00'!
getNextKeyValue: aFileStream
"Return an association with the next key/value pair found in aFileStream, or nil. Go through the stream line by line until the next key is found. The stream is scanned for the next line containing an $=. The key is the part of that line string before the $=, the value is the part after. If the end of the file or next topic is reached before finding a key/value, then Return nil. Assumes aFileStream positioned after topic string of desired topic"

	|   nextKey  string  eqIndex   |

	"Make sure the stream is not at the end"
	aFileStream atEnd ifTrue: [ ^nil ].

	nextKey _ nil.
	[
			 "Make sure the next line does not define a topic"
			(aFileStream peek = $[) ifTrue: [ ^nil ].

			string _ aFileStream nextLineForConfigFile.
			"Keep searching if:
				1. string is empty
				2. The first char is not alphanumeric
				3. an = sign is hit before the end of the line"
			string isEmpty 
				ifTrue: [ ^self getNextKeyValue: aFileStream ].
			( 
				(string first isAlphaNumeric) and: [
				(eqIndex _ string indexOf: $=) ~= 0 
			])
					ifFalse: [ ^self getNextKeyValue: aFileStream ].

			"A key has been found.  Create an assoc w/ key-value strings"
			nextKey _ Association
				key:  (string copyFrom: 1 to: (eqIndex - 1))
				value: (string copyFrom: (eqIndex + 1) to: string size)
	]
		on: Error
		do: [ :ex | ex return ].

	^nextKey! !

!ConfigFile methodsFor: 'private' stamp: 'sps 10/5/2001 17:58'!
getNextTopic: aFileStream
"Return the next 'Topic' found in the stream, or nil. The topic is the string found enclosed between square brackets (ie. [This Is A Topic]). The topic string must be on it's own line in the file"

	|  nextTopic string |	

	aFileStream atEnd ifTrue: [ ^nil ].

	nextTopic _ 	nil.
	[			
			"Get the next line"
			string _ aFileStream nextLineForConfigFile withBlanksTrimmed.

			"Keep searching if string isEmpty or if it's not encloseed in $[ and $]"
			string isEmpty
				ifTrue:[^self getNextTopic: aFileStream ].
			((string first == $[) and: [ string last == $] ])
				ifFalse: [ ^self getNextTopic: aFileStream ].

			"It is the beginning of a topic name, return the string after [ and before ] "
			nextTopic _ string copyFrom: (string indexOf: $[) + 1 to: (string indexOf: $]) - 1
	]	
		on: Error
		do: [:ex | ex return].

	^nextTopic! !

!ConfigFile methodsFor: 'private' stamp: 'sps 9/28/2001 15:38'!
isTopicThere: aTopicString onStream: aFileStream
"Return whether a topic called aTopicString is found on aFileStream."

	| topic |

	(topic _ self getNextTopic: aFileStream) 
		ifNil:[ ^false ].

	(topic = aTopicString) 
		ifTrue: [ ^true ].

	^self isTopicThere: aTopicString onStream: aFileStream! !

!ConfigFile methodsFor: 'private' stamp: 'sps 9/28/2001 15:45'!
privateSetKey: aKeyString topic: aTopicString value: aValueString
"Set the key to aValueString for aTopicString, both in the topicDict cache and in the file itself.  If Topic or key doesn't exist, create it"

	| fileStream topicIndex keyValueAssoc |
	fileStream _ self fileStream.

	[ 
		"First find the topic index"
		(self isTopicThere: aTopicString onStream: fileStream) ifFalse: [ 
			"Topic did not exist -- write both topic and key/value"
			self addTopic: aTopicString onStream: fileStream.
			self addKey: aKeyString value: aValueString onStream: fileStream.
		] ifTrue:[
			"Now find the key index. Save the topic index first 
			(go back to it if no key is found)"
			topicIndex _ fileStream position.
			keyValueAssoc _ self findKeyValuePair: aKeyString onStream: fileStream.
			keyValueAssoc isNil ifTrue: [
				"Key did not exist. Move the stream back, then add the key after the topic string"
				fileStream position: topicIndex.
				self addKey: aKeyString value: aValueString onStream: fileStream.
			] ifFalse:[
				"Both the topic and the key where already there. Rewrite the value"
				self rewriteKey: keyValueAssoc newValue: aValueString onStream: fileStream.
			].
		].
	] ensure: [ 
			fileStream flush; close. 
			self cacheSetKey: aKeyString topic: aTopicString value: aValueString.
	].! !

!ConfigFile methodsFor: 'private' stamp: 'sps 9/28/2001 15:41'!
readFromFile
"Read in the config file and cache all the values in the topicDict"

	| fileStream topic curTopicDict keyVal |

	self topicDict: Dictionary new.
	fileStream _ self fileStream.
	[ 
		[ (topic _ self getNextTopic: fileStream) isNil ]  whileFalse:[

			self topicDict at: topic put: (curTopicDict _ Dictionary new).

			[ (keyVal _ self getNextKeyValue: fileStream) isNil ] whileFalse: [
				curTopicDict add: keyVal.	
			]
		]
	] ensure: 
		[ 
			fileStream close. 
			^false 
		].

	fileStream close.
	^self topicDict! !

!ConfigFile methodsFor: 'private' stamp: 'sps 9/28/2001 15:39'!
rewriteKey: aKeyAssoc newValue: aValueString onStream: aFileStream
		"Rewrite the key/value line for aKeyString.
		Assumes that aFileStream is currently at the end of the key/Value line"

	| startOfLine endOfLine |

	"Move the stream back over the cr"
	aFileStream previous.
	"The next line skips back to either the beginning of the key/value line or to the 2nd char on the
	line (possible due to cr v. cr/lf discrep.). No matter -- We'll make sure to move back before the = sign"
	aFileStream skip: -1 * ( aKeyAssoc key size + aKeyAssoc value size + 2).
	"Now move up to the equal sign"
	[ aFileStream next == $= ] whileFalse: [ ].

	endOfLine _ aFileStream position + aKeyAssoc value size.
	startOfLine _ aFileStream position.
	aFileStream 
		spliceString: (self configFileForm: aValueString)
		from: startOfLine
		to: endOfLine.

	^true! !

!ConfigFile methodsFor: 'private' stamp: 'sps 9/28/2001 15:39'!
saveOutTopicDict
		"Save out the topicDict from filename"

	"Iterate through the Topics. For each topic, iterate through the keys
	and save out the topic/key/value pair"

	| value newCfg tempName |

	tempName _ self localFileName, '.temp'.
	newCfg _ ConfigFile fileName: tempName directory: self directory createIfNotThere: true.

	self topicDict keysDo: [ :topic | "Go through the topics"
		(self topicDict at: topic) keysDo: [ :key | "Go through that topic's keys"
			value _ (self topicDict at: topic) at: key. "Set the key"
			newCfg setKey: key topic: topic value: value.
		]
	].

	self directory deleteFileNamed: self localFileName.
	self directory rename: tempName toBe: self localFileName
! !

!ConfigFile methodsFor: 'private'!
topicDict
	"topicDict contains topics as the keys. 
	Each value is another dict with the key/value pairs for the topic"

	^topicDict! !

!ConfigFile methodsFor: 'private' stamp: 'sps 9/28/2001 15:39'!
topicDict: aDictionary 
	"topicDict contains topics as the keys. 
	Each value is another dict with the key/value pairs for the topic"

	^topicDict _ aDictionary! !

!ConfigFile methodsFor: 'initialization' stamp: 'sps 9/28/2001 15:42'!
initialize
		"Read in the file to create the topicDict"

	self readFromFile.! !

!ConfigFile methodsFor: 'initialization'!
reread
		"Throw out the current cache, read in a new one"

	self topicDict: nil.
	^self initialize! !


!ConfigFile class methodsFor: 'instance creation' stamp: 'sps 9/26/2001 22:29'!
fileName: fileName
"If fileName is a full pathname, then use that as a config file. If not, use it as the name of a config file in FileDirectory>>default"

	| pathNameLength pathName shortFileName |

	(fileName includes: FileDirectory pathNameDelimiter) ifTrue:[
		pathNameLength _ 
			fileName size
			- (fileName reversed findString: FileDirectory slash) 
			+ 1.
		shortFileName _ fileName copyFrom: pathNameLength + 1 to: fileName size.
		pathName _ fileName copyFrom: 1 to: pathNameLength.
	] ifFalse:[
		shortFileName _ fileName.
		pathName _ FileDirectory default pathName.
	].

	^self fileName: shortFileName directoryName: pathName createIfNeeded: true! !

!ConfigFile class methodsFor: 'instance creation' stamp: 'sps 9/28/2001 15:39'!
fileName: aFilenameString directory: directory createIfNotThere: bCreate
"Create a ConfigFile on aFilenameString in the directory. If bCreate is true, than create the file if it isn't already there"

	| newInstance fullPathName separator |

	"Make sure the directory is there"
	((FileDirectory default) directoryExists: directory pathName) ifFalse: [
		MessageBox message: 
			('Cannot create config file. Could not find directory: ', directory pathName).
			
		^nil 
	].

	"Construct the full pathname"
	(directory pathName last == FileDirectory pathNameDelimiter)
		ifTrue: [ separator _ '' ]
		ifFalse: [ separator _ FileDirectory slash ].
	fullPathName _ directory pathName, separator, aFilenameString.

	"Create it if it doesn't exist and bCreate is true"
	(FileDirectory new fileExists: fullPathName)ifFalse: [ 
		bCreate	ifFalse: [
				^nil 
		]ifTrue: [
			"Create the file"
			(FileStream newFileNamed: fullPathName) close.
		]
	].

	"Create the new instance of self, initialize"
	newInstance _ self new.
	newInstance 
		localFileName: fullPathName;
		directory: directory.
	newInstance initialize.
	^newInstance! !

!ConfigFile class methodsFor: 'instance creation' stamp: 'sps 9/26/2001 22:51'!
fileName: aFilenameString directoryName: dirName createIfNeeded: bCreate
"Create a ConfigFile on aFilenameString in the directory dirName. If bCreate is true, than create the file if it isn't already there"

	| aDirectoryString |

	(dirName last == FileDirectory pathNameDelimiter)
		ifTrue:[ aDirectoryString _ dirName copyFrom: 1 to: (dirName size - 1) ]
		ifFalse:[ aDirectoryString _ dirName ].

	^self 
		fileName: aFilenameString 
		directory: (FileDirectory on: aDirectoryString)
		createIfNotThere: bCreate.
! !


!PositionableStream methodsFor: 'sps support' stamp: 'sps 10/5/2001 18:09'!
nextLineForConfigFile
"Make sure it goes past both cr and lf"

	| line  nextChar |
  line := self nextLine.
  self atEnd ifFalse: [
		(((nextChar _ self peek) = Character lf) or: [ nextChar = Character cr])
			ifTrue: [ self next ].
  ].
  ^line! !

!PositionableStream methodsFor: 'sps support' stamp: 'sps 9/28/2001 15:27'!
previous
"Return the last element read"
	
	self position <= 0
			ifTrue:[ ^nil ].
	self position: (self position - 1).
	^self next
	! !

!PositionableStream methodsFor: 'sps support' stamp: 'sps 9/26/2001 18:51'!
spliceString: aString from: start to: stop
	"SPS: Splice aString into self, replacing contents from startIndex to stopIndex.
	If the stream has shrunk, it is padded with spaces"

	| spliceString |

	"Make sure startIndex and stopIndex are legal first"
	start <= stop ifFalse: [ ^nil ].

	"Save the end of the stream in spliceString."
	self position: stop.
	(spliceString := WriteStream on: String new).
	[ self atEnd ] whileFalse: [ spliceString nextPut: self next ].

	"Go back and write over from the start position"
	self
		position: start;
		nextPutAll: aString;
		nextPutAll: spliceString contents.

	"Pad end w/ spaces if necessary"
	[ self atEnd ] whileFalse: [ self nextPut: $  ].! !


!PositionableStream reorganize!
('accessing' contents contentsOfEntireFile last next: next:into: next:into:startingAt: next:putAll: next:putAll:startingAt: nextDelimited: nextInto: nextInto:startingAt: nextLine nextWordsInto: originalContents peek peekFor: upTo: upToAll: upToEnd)
('testing' atEnd isBinary isEmpty)
('positioning' match: padTo:put: padToNextLongPut: position position: reset resetContents setToEnd skip: skipTo:)
('fileIn/Out' checkForPreamble: command: copyMethodChunkFrom: copyMethodWithPreamble:from: copyPreamble:from: decodeStyle:version: fileIn fileInAnnouncing: header nextChunk nextChunkText skipSeparators skipSeparatorsAndPeekNext skipStyleChunk trailer unCommand verbatim:)
('private' on: positionError setFrom:to:)
('nonhomogeneous accessing' nextInt32 nextInt32Put: nextLittleEndianNumber: nextLittleEndianNumber:put: nextNumber: nextNumber:put: nextString nextStringOld nextStringPut: nextWord nextWordPut:)
('converting' asBinaryOrTextStream asZLibReadStream)
('sps support' nextLineForConfigFile previous spliceString:from:to:)
!

"Postscript:
Show the class comment for ConfigFile."

(Workspace new contents: ConfigFile comment asString) openLabel: 'Class comment for ConfigFile'. 

!

